using System;
using System.IO;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using System.Reflection;
using System.Runtime.Remoting;
using System.Runtime.InteropServices;
using System.Threading;
using System.Web.Services.Protocols;
using EnvDTE;
using Extensibility;
using Microsoft.Office.Core;
using VSUserControlHostLib;
using SHDocVw;

namespace DCL_VSI
{
	public class DCL_VSI
	{
		//Attributes
		private DTE applicationObject;
		private AddIn addInInstance;
		private Settings settings;
		private string[] allExperiments = null;		
		private JobExplorer jobExp;
		private CommandBar dclCmdBar = null;
		private CommandBar dclToolBar = null;
		private CommandBar cameraCmdBar = null;		
		private ArrayList baseCommands = null;
		private ArrayList sendToCommands = null;
		private ArrayList cameraViewCommands = null;
		private ArrayList toolBarItems = null;
		private CommandBarComboBox sendToDropDown = null;
		private Window jobExplorerWindow = null;
		private string jobExpWinCLSID = "{C19A9E2F-4AD9-492c-8494-270B5B5DA8D4}";
		private System.Threading.Thread refreshMenuAndJobExplorerThread;


		public DCL_VSI(AddIn addInInst, DTE application)
		{
			addInInstance = addInInst;
			applicationObject = application;
			settings = new Settings();
			baseCommands = new ArrayList();
			sendToCommands = new ArrayList();
			cameraViewCommands = new ArrayList();
			toolBarItems = new ArrayList();
		}
		
		public void startup()
		{			
			// Create base menu items
			setBaseCommands();			
			
			// Create dcl toolbar
			createToolBar();

			// Create tool window 'Job Explorer'
			createJobExplorer();
			setJobExplorerTabPicture();
			jobExp.DCL_VSI = this;
			
			// Initialize settings
			string  registryRoot = applicationObject.RegistryRoot;
			settings.startup(this, registryRoot);

			// Initialze 'Job Explorer'
			jobExp.setServer(settings);
		}

		private void createJobExplorer()
		{
			try
			{
				object objTemp = null;
				jobExplorerWindow = applicationObject.Windows.CreateToolWindow (addInInstance,
					"VSUserControlHost.VSUserControlHostCtlDCL",
					"Job Explorer",
					jobExpWinCLSID,
					ref objTemp);
				
				// When using the hosting control, you must set visible to true before calling
				// HostUserControl, otherwise the UserControl cannot be hosted properly.				
				jobExplorerWindow.Visible = true;
				
				Assembly asm = Assembly.GetExecutingAssembly();

				IVSUserControlHostCtl objControl;
				objControl = (IVSUserControlHostCtl)objTemp;
				// Loads the JobExplorer(UserControl) in VSUserControlhost
				objControl.HostUserControl(asm.Location, "DCL_VSI.JobExplorer");
				// Ask the VSUserControlHost for the ObjectHandle of the hosted control					
				ObjectHandle jobExpObjHandle = (ObjectHandle) objControl.GetObjectHandle();
				//Unwrap the object handle to get the 'JobExplorer object' hosted in VSUserControl
				jobExp = (JobExplorer) jobExpObjHandle.Unwrap();					
			}
			catch(System.Exception e)
			{	
				Error.notify("Error while creating Job Explorer.\n" + Error.FatalErrorInfo);
				Error.log(e);
			}
		}
		private void setJobExplorerTabPicture()
		{
			Assembly asm = Assembly.GetExecutingAssembly();
			
			string tabPicturePath = asm.Location.Substring(0,asm.Location.LastIndexOf('\\') + 1) + "JobExplorer.ico";
								
			if (File.Exists(tabPicturePath))
			{
				Image tabImage = Image.FromFile(tabPicturePath);					
					
				object tabPicture = ImageConverter.getIPictureDisp(tabImage);					
					
				try
				{					
					jobExplorerWindow.SetTabPicture(tabPicture);
					
					tabImage.Dispose();
				}
				catch (Exception e)
				{
					Error.log(e);
				}
			}
			else
			{
				Error.log("DCL_VSI", "startup()", "File \"JobExplorer.ico\" not found.");
			}
		}

		private Command createCommand(string name, string caption, string toolTip, int iconFaceID, bool separator, CommandBar cmdBar)
		{
			// Try to delete menu item (command), if it exists
			try
			{
				applicationObject.Commands.Item("DCL_VSI.Connect." + name, 1).Delete();
			}
			catch {}
			// Create new menu item (command)
			object []contextGUIDS = new object[] {};
			Command cmd = null;			
			try
			{			
				cmd = applicationObject.Commands.AddNamedCommand(addInInstance,
					name,
					caption,
					toolTip,
					true,
					iconFaceID,
					ref contextGUIDS,
					(int)vsCommandStatus.vsCommandStatusSupported+(int)vsCommandStatus.vsCommandStatusEnabled);
				
				// Assign command to menu (command bar)
				CommandBarControl cmdBarControl = null;
							
				cmdBarControl = cmd.AddControl(cmdBar, 1);
				// Create menu item separator
				if (separator == true)
				{
					cmdBarControl.BeginGroup = true;
				}
			}
			catch(System.Exception e)
			{	
				Error.notify("Error while creating \"" + caption + "\" menu item." + Error.FatalErrorInfo);
				Error.log(e);
			}
			
			return cmd;
		}
		
		private void setBaseCommands()
		{
			try
			{
				// Create 'DCL' main menu item
				CommandBar MainMenuBar = (CommandBar)applicationObject.CommandBars["MenuBar"];				
				CommandBarControl dclMenuBarControl;
				dclMenuBarControl = MainMenuBar.Controls.Add(MsoControlType.msoControlPopup,Type.Missing,Type.Missing,21,true);
				dclMenuBarControl.Caption = "&DCL";
				dclMenuBarControl.Visible = true;

				dclCmdBar = ((CommandBarPopup)dclMenuBarControl).CommandBar;
			
				// Create base commands			
				baseCommands.Add( createCommand("showSettings",
					"Settings...",
					"Shows Settings Dialog",
					222,
					true,
					dclCmdBar) );			
				baseCommands.Add( createCommand("showJobExplorer",
					"Show Job Explorer",
					"Shows Job Explorer",
					461,
					true,
					dclCmdBar) );
				baseCommands.Add( createCommand("refreshMenuAndJobExplorer",
					"Refresh",
					"Refreshs Menu and Job Exlorer",
					1020,
					true,
					dclCmdBar) );
			}
			catch(System.Exception e)
			{	
				Error.notify("Error while creating \"DCL\" menu." + Error.FatalErrorInfo);
				Error.log(e);
			}
		}

		private void setSendToCommands()
		{	
			// Create 'Send To ...' menu items
			int i = 0;
			foreach (string experimentName in allExperiments)
			{
				//parameter and command name are separated by '0'(another character was not possible)
				sendToCommands.Add(createCommand("sendTo0" + i.ToString(),
					"Send to " + experimentName,
					"Sends current program (job) to experiment " + experimentName,
					0,
					false,
					dclCmdBar));
				i++;
			}			
		}

		private void setCameraViewCommands(ArrayList cameraNames)
		{
			
			if (cameraNames != null)
			{	
				// Create 'Camera Views' submenu
				CommandBarControl cameraMenuBarControl = null;
				try
				{
					cameraMenuBarControl = dclCmdBar.Controls.Add(MsoControlType.msoControlPopup,Type.Missing,Type.Missing,1,true);
					cameraMenuBarControl.Caption = "Camera Views";
					cameraMenuBarControl.BeginGroup = true;
					cameraMenuBarControl.Visible = true;
				
					cameraCmdBar = ((CommandBarPopup)cameraMenuBarControl).CommandBar;
			
					// Create submenu items
					int i = 0;			
					foreach (string cameraName in cameraNames)
					{						
						//parameter and command name are separated by '0'(another character was not possible)
						cameraViewCommands.Add(createCommand("CameraView0" + i.ToString(),
							cameraName,
							"Show Camera " + cameraName,
							0,
							false,
							cameraCmdBar));
						i++;
					}
				}
				catch (Exception e)
				{
					Error.notify( "Error while creating \"Show Camera\" menu" + Error.FatalErrorInfo);
					Error.log(e);
				}

			}
		}		
		
		private void createToolBar()
		{
			try
			{				
				dclToolBar = applicationObject.CommandBars["DCL ToolBar"];				
			}
			catch
			{
				dclToolBar = (CommandBar)applicationObject.Commands.AddCommandBar("DCL ToolBar",
					vsCommandBarType.vsCommandBarTypeToolbar,
					null,
					0);
				dclToolBar.Position = MsoBarPosition.msoBarTop;
			}
			dclToolBar.Enabled = false;
		}

		private void addToolBarItems()
		{
			try
			{
				CommandBarControl sendToButtonControl = dclToolBar.Controls.Add(MsoControlType.msoControlButton,
					Type.Missing,
					Type.Missing,
					1,
					true);
				toolBarItems.Add(sendToButtonControl);

				CommandBarButton sendToButton = (CommandBarButton)sendToButtonControl;

				sendToButton.FaceId = 1924;
				sendToButton.Caption = "Send to";
				sendToButton.Style = MsoButtonStyle.msoButtonIconAndCaption;
				sendToButton.Click += new _CommandBarButtonEvents_ClickEventHandler(sendToButton_OnClick);
				sendToButton.Visible = true;

				CommandBarControl sendToDropDownControl = dclToolBar.Controls.Add(MsoControlType.msoControlDropdown,
					Type.Missing,
					Type.Missing,
					2,
					true);
				toolBarItems.Add(sendToDropDownControl);
				
				sendToDropDown = (CommandBarComboBox)sendToDropDownControl;
				sendToDropDown.Width = 160;
				
				foreach (string experimentName in allExperiments)
				{
					((CommandBarComboBox)sendToDropDownControl).AddItem(experimentName, Type.Missing);				
				}

				sendToDropDown.ListIndex = 1;
				sendToDropDown.Visible = true;
				
				dclToolBar.Enabled = true;
				dclToolBar.Visible = true;
			}
			catch (Exception e)
			{
				Error.notify("Error while creating toolbar item." + Error.FatalErrorInfo);
				Error.log(e);
			}
		}

		public void deleteBaseCommands()
		{
			try
			{
				foreach (Command cmd in baseCommands)
				{
					cmd.Delete();
				}
			}
			catch (Exception e)
			{
				Error.log(e);
			}
		}
		
		public void deleteSendToCommands()
		{
			try
			{
				foreach (Command cmd in sendToCommands)
				{					
					cmd.Delete();
				}
				sendToCommands.Clear();				
			}
			catch (Exception e)
			{
				Error.log(e);
			}
		}

		public void deleteCameraViewCommands()
		{
			try
			{
				if (cameraCmdBar != null)
				{
					foreach (Command cmd in cameraViewCommands)
					{					
						cmd.Delete();
					}
					cameraViewCommands.Clear();

					dclCmdBar.Controls["Camera Views"].Delete(true);
					cameraCmdBar = null;
				}
			}
			catch (Exception e)
			{
				Error.log(e);
			}
		}

		public void deleteToolBarItems()
		{
			try
			{
				foreach (CommandBarControl cmdBarCtl in toolBarItems)
				{
					cmdBarCtl.Delete(true);
				}				
				sendToDropDown = null;

				toolBarItems.Clear();
			}
			catch (Exception e)
			{
				Error.log(e);
			}			
		}


		void sendToButton_OnClick(CommandBarButton btn, ref bool someBool)
		{
			sendTo(sendToDropDown.ListIndex - 1);
		}

		public void sendTo(int experiment)
		{
			StartSendToThread startSendToThread = new StartSendToThread(this, experiment);
			
			try
			{
				System.Threading.Thread t = new System.Threading.Thread (new ThreadStart(startSendToThread.start));
				t.Name = "Thread SendTo Experiment: " + experiment.ToString();
				t.Start();
			}
			catch (ThreadAbortException){}
		}

		private class StartSendToThread
		{
			private int experiment;
			private DCL_VSI dclvsi;

			public StartSendToThread(DCL_VSI dv, int exp)
			{
				experiment = exp;
				dclvsi = dv;
			}
			public void start()
			{
				dclvsi.sendToThread(experiment);
			}
		}

		private void sendToThread(int experiment)
		{
			System.Threading.Thread currentThread;
			Document activeDocument = null;
			string code = null;
			string experimentName;
			string ticket = null;

			// Get executing thread
			currentThread = System.Threading.Thread.CurrentThread;

			// Find active document
			try
			{
				Window activeWindow = applicationObject.ActiveWindow;

				activeDocument = applicationObject.ActiveDocument;

				if (activeWindow.Kind == "Document")
				{					

					if (activeDocument.Kind != Constants.vsDocumentKindText)
					{
						Error.notify("Invalid document type. Sending to DCL canceled.");
						currentThread.Abort();
					}
				}
				else
				{
					if (activeDocument == null)
					{
						Error.notify("No document open. Sending to DCL canceled.");
					}
					else
					{
						Error.notify("A tool window is currently selected. Sending to DCL canceled.\n\n" +
							"Please select a document.");
						activeDocument.Activate();
					}
					currentThread.Abort();
				}
			}
			catch(ThreadAbortException) {}
			catch(Exception e)
			{	
				Error.notify("Unable to send active document. Sending to DCL canceled.");
				Error.log(e);
				currentThread.Abort();
			}

			// Save active document
			try
			{
				vsSaveStatus saveStatus;
				saveStatus = activeDocument.Save("");
				if (saveStatus == vsSaveStatus.vsSaveCancelled) 
				{
					Error.notify("Unable to save active document. Sending to DCL canceled.");
					currentThread.Abort();
				}
			}
			catch(ThreadAbortException) {}
			catch(Exception e)
			{	
				Error.notify("Unable to save active document. Sending to DCL canceled.");
				Error.log(e);
				currentThread.Abort();			
			}

			// Get full filename and open file
			string filename;
			try
			{
				filename = activeDocument.FullName;
				StreamReader SR = File.OpenText(filename);
				code = SR.ReadToEnd();
				SR.Close();
			}
			catch (Exception e)
			{
				Error.notify("Unable to read active document. Sending to DCL canceled.");
				Error.log(e);
				currentThread.Abort();		
			}

			// Get mutex
			jobExp.JobExplorerLock.WaitOne();

			// Get name of experiment
			experimentName = allExperiments[experiment];
			
			// Get new ticket
			try
			{
				ticket = settings.TicketServer.GetNewTicket(settings.Username, settings.Password);
			}
			catch (SoapException e)
			{
				Error.handleSoapException("Sending to DCL canceled.",e);
				jobExp.JobExplorerLock.ReleaseMutex();
				currentThread.Abort();
			}
			catch (Exception e)
			{
				Error.handleConnectionException("Sending to DCL canceled.", e);
				jobExp.JobExplorerLock.ReleaseMutex();
				currentThread.Abort();		
			}
			if (ticket == "S_ERROR_INVALID_LOGIN")
			{
				Error.notify("Invalid username or password. Sending to DCL canceled.");
				jobExp.JobExplorerLock.ReleaseMutex();
				currentThread.Abort();
			}
            
			// Send code to DCL
			try
			{
				string result;
				result = settings.ExperimentService.UseExperiment(ticket, experimentName, code);

				if (result != "S_OK")
				{
					try 
					{
						settings.TicketServer.FreeTicket(ticket);
						settings.ExperimentService.FreeTicket(ticket);
					}
					catch {}
					Error.notify("DCL is currently unable to process your request. Sending to DCL canceled");
					Error.log("DCL_VSI", "sendToThread", "Result of function call UseExperiment(): " + result);
					jobExp.JobExplorerLock.ReleaseMutex();
					currentThread.Abort();
				}
			}
			catch(ThreadAbortException) {}
			catch (SoapException e)
			{
				try 
				{
					settings.TicketServer.FreeTicket(ticket);
					settings.ExperimentService.FreeTicket(ticket);
				}
				catch {}
				Error.handleSoapException("Sending to DCL canceled.", e);
				jobExp.JobExplorerLock.ReleaseMutex();
				currentThread.Abort();
			}
			catch (Exception e)
			{
				try 
				{
					settings.TicketServer.FreeTicket(ticket);
					settings.ExperimentService.FreeTicket(ticket);
				}
				catch {}
				Error.handleConnectionException("Sending to DCL canceled.", e);
				jobExp.JobExplorerLock.ReleaseMutex();
				currentThread.Abort();
			}			

			try
			{
				jobExp.StatusBarMsg = "Loading...";
				
				if (jobExp.IsRefreshing == false)
				{
					TreeNode tn = jobExp.addJob(ticket, true);

					jobExp.StatusBarMsg = "Ready";

					jobExp.selectJobNode(tn);				
				
					// Release mutex
					jobExp.JobExplorerLock.ReleaseMutex();				
				
					// Show results in 'Output' window
					jobExp.viewCompilerOutput(tn);
				}
				else
				{
					// Release mutex
					jobExp.JobExplorerLock.ReleaseMutex();
				}

			}
			catch(ThreadAbortException){
				jobExp.JobExplorerLock.ReleaseMutex();
			}
			catch (SoapException e)
			{
				jobExp.StatusBarMsg = "Error while adding job";
				Error.handleSoapException("Error while adding job to Job Explorer.", e);
				jobExp.JobExplorerLock.ReleaseMutex();
			}
			catch (Exception e)
			{
				jobExp.StatusBarMsg = "Error while adding job";
				if (e.Message.StartsWith(Error.ConnectionError) == false)
				{
					Error.log(e);
				}
				jobExp.JobExplorerLock.ReleaseMutex();
			}					
		}

		public void CameraView(int camera)
		{
			CameraView cameraView = new CameraView((string)settings.CameraUrls[camera]);
			cameraView.Text = (string)settings.CameraNames[camera];
			cameraView.Show();			
		}

		/* 'param' is unused; but the parameter exists, because we want to simplify reflection
		   see class: connect / method: Exec() */
		public void refreshMenuAndJobExplorer(int param)
		{
			
			jobExp.StatusBarMsg = "Starting Refresh...";
			
			deleteSendToCommands();
			deleteCameraViewCommands();
			deleteToolBarItems();

			jobExp.clear();

			// Shows Job Explorer Window
			jobExplorerWindow.Visible = true;
			
			try
			{
				refreshMenuAndJobExplorerThread.Abort();
			}
			catch{}
			
			try
			{
				refreshMenuAndJobExplorerThread = new System.Threading.Thread(new ThreadStart(refreshThread));
				refreshMenuAndJobExplorerThread.Start();
			}
			catch(ThreadAbortException){}
		}

		private void refreshThread()
		{
			if (settings.checkConnection()==null)
			{				
				// Create 'Camera Views' submenu items
				setCameraViewCommands(settings.CameraNames);
				try
				{
					// Create 'Send To ...' menu items
					allExperiments = settings.ExperimentService.ListExperiments();
					setSendToCommands();
					
					// Add toolbar items
					addToolBarItems();
				}
				catch (ThreadAbortException) {}
				catch (SoapException e)
				{
					deleteCameraViewCommands();
					Error.handleSoapException("Refresh of Menu failed.", e);
					System.Threading.Thread.CurrentThread.Abort();
				}
				catch (Exception e)
				{
					deleteCameraViewCommands();
					Error.handleConnectionException("Refresh of Menu failed.", e);
					System.Threading.Thread.CurrentThread.Abort();
				}
				// Load jobs of user in Job Explorer tool window
				jobExp.refreshJobs();
			}
			else
			{
				jobExp.StatusBarMsg = "No Connection";
			}
		}

		/* 'param' is unused; but the parameter exists, because we want to simplify reflection
		   see class: connect method: Exec() */
		public void showJobExplorer(int param)
		{
			jobExplorerWindow.Visible = true;			
		}		

		/* 'param' is unused; but the parameter exists, because we want to simplify reflection
		   see class: connect method: Exec() */
		public void showSettings(int param)
		{
			settings.ShowDialog();
		}

		public void printToOutputWindow (string text, bool append)
		{
			Window win;
			OutputWindow outputWindow = null;
			OutputWindowPane outputPane = null;
			
			try
			{
				// Get the OutputWindow 
				win = applicationObject.Windows.Item(Constants.vsWindowKindOutput);
				win.Activate();
				outputWindow = (OutputWindow) win.Object;
			}
			catch (ThreadAbortException) {}
			catch (Exception e)
			{
				Error.notify("Error while printing to 'Output' window.");
				Error.log(e);
			}


			// Find or create OutputPane
			try
			{
				outputPane = outputWindow.OutputWindowPanes.Item("DCL - Compiler Output");
			}
			catch (ThreadAbortException) {}
			catch 
			{
				outputPane = outputWindow.OutputWindowPanes.Add ("DCL - Compiler Output");
			}
			if (append == false)
			{
				outputPane.Clear();
			}
			
			// Write text to OutputPane
			outputPane.OutputString(text);
			outputPane.Activate();			
		}

		public void openNewCodeWindow (string filename, string code)
		{
			Window win;
			Document doc;
			TextDocument textDoc;
			EditPoint editPoint;

			try
			{
				// Create new text document
				win = applicationObject.ItemOperations.NewFile( "General\\Text File" , filename, Constants.vsViewKindCode);
				
				// Get the TextDocument-Object of the new file
				doc = win.Document;
				doc.Activate();
				textDoc = (TextDocument) doc.Object("TextDocument");
				
				// Get an EditPoint-object in the TextDocument
				editPoint = textDoc.CreateEditPoint(textDoc.StartPoint);

				// Write to the TextDocument
				editPoint.Insert(code);
			}
			catch (Exception e)
			{
				Error.notify ("Unable to create local code file.\n" + Error.FatalErrorInfo);
				Error.log(e);
			}
		}

		public void showInternetExplorer (string url)
		{
			// Open an internet-explorer window
			// Get the browser window.
			Window webBrowserWindow = applicationObject.Windows.Item(Constants.vsWindowKindWebBrowser);
			
			// Force the window to be visible.
			webBrowserWindow.Visible = true ;
			
			// Get the IE control inside the browser window.
			SHDocVw.WebBrowser webBrowser = null;
			try
			{
				webBrowser = (SHDocVw.WebBrowser)webBrowserWindow.Object;
			}
			catch(System.Exception e)
			{	
				Error.notify("Unable to show results in internet explorer. " + Error.FatalErrorInfo);
				Error.log(e);
			}			
            
			Object zero = 0 ;
			Object emptyString = "" ;				
			// Navigate to the Url
			webBrowser.Navigate (url, ref zero, ref emptyString, ref emptyString, ref emptyString);

		}
	}
}
