using System;
using System.IO;
using System.Reflection;
using System.Windows.Forms;

namespace DCL_VSI
{	
	public class Error
	{
		private static string errorFileName = null;
		private const string title = "Error in DCL VSI";
		public const string ConnectionError = "The underlying connection was closed:";
		public const string FatalErrorInfo = "For further information see \"error.log\" or contact the OSM-group at the HPI Potdam.";

		public Error()
		{		
		}

		public static void setFileName()
		{
			if (errorFileName == null)
			{
				// Set path and filename of the installed error.log
				Assembly asm = Assembly.GetExecutingAssembly();	
				errorFileName = asm.Location.Substring(0,asm.Location.LastIndexOf('\\') + 1) + "error.log";
			}
		}

		public static void notify(string userMessage)
		{
			// Shows a messagebox without logging
			MessageBox.Show(userMessage, title, MessageBoxButtons.OK, MessageBoxIcon.Error);
		}

		public static void log(Exception e)
		{
			// Log the error
			string[] errorText = new string[4];
			errorText[0] = "Message: " + e.Message;
			errorText[1] = "Application/Object: " + e.Source;
			errorText[2] = "Method: " + e.TargetSite;
			errorText[3] = "StackTrace:\n" + e.StackTrace;
			
			writeLog(errorText);
		}


		public static void log(string className, string methodName,	string internalDescription)
		{
			// Log the error
			string[] errorText = new string[3];
			errorText[0] = "Classname: " + className;
			errorText[1] = "Methodname: " + methodName;
			errorText[2] = "Internal description: " + internalDescription;

			writeLog(errorText);
		}

		public static void handleSoapException(string userMessage, Exception e)
		{
			notify("DCL is currently unable to process your request.\n" + userMessage);
			log(e);
		}

		public static void handleConnectionException(string userMessage, Exception e)
		{
			if (e.Message.StartsWith(ConnectionError) == true)
			{
				notify("Connection to DCL was closed. Please check your VPN.\n" + userMessage);
			}
			else
			{
				notify("Unkonwn error occurred.\n" + userMessage + "\n" + FatalErrorInfo);
				log(e);
			}
		}

		private static void writeLog(string[] errorText)
		{
			
			// Write errorText to logfile
			// Only one thread can write to error.log at the same time
			lock (typeof(Error))
			{
				DateTime now = DateTime.Now;
				setFileName();
				try
				{
					StreamWriter w = null;

					if (File.Exists(errorFileName)) 
					{
						// If error.log exists, append to the file
						w = File.AppendText(errorFileName);
					}
					else
					{
						// If no error.log exists, create a new one
						w = File.CreateText(errorFileName);
					}
					
					// Write current date, time and the error to the file
					w.WriteLine();
					w.WriteLine("---------- " + now.ToShortDateString() +" - " + now.ToLongTimeString() + " ----------------------------");
					foreach (string i in errorText)
					{
						w.WriteLine(i);
					}
					w.WriteLine("-------------------------------------------------------------");
					w.WriteLine();
					w.WriteLine();
					
					// Close the file
					w.Close();
				}
				catch
				{
					MessageBox.Show("An error occurred!\n" +
						"Unable to write to error.log.",
						"title",
						MessageBoxButtons.OK,
						MessageBoxIcon.Error);
				}
			}
		}
	}
}
