using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Threading;
using System.Web.Services.Protocols;
using DCL_VSI.de.discourse.was;
using DCL_VSI.de.discourse.was1;
using Crownwood.Magic.Menus;



namespace DCL_VSI
{
    
	public class JobExplorer : System.Windows.Forms.UserControl
	{
		//Attributes
		private DCL_VSI dclvsi;
		private Settings settings;
        
		private DCLTicketServer ticketServer;
		private DCLExperimentService experimentService;
        
		private Mutex jobExplorerLock;
        
		private Hashtable usedExperiments;
		private int experimentImage = 7;
        
		private TreeView jobs;
		enum JobStatusType {Unknown = 0, Queued = 1, Compilation = 2, Execution = 3, Failed = 4, Canceled = 5, Finished = 6}
		private Hashtable convertToJobStatusType;
		private Hashtable jobStatusCaption;
		private Hashtable resultCode;
		private Font jobFont;
		private System.Threading.Thread updateJobStatusThread = null;
		private bool isRefreshing = false;
		private System.Threading.Thread compilerOutputThread = null;

		private int openCodeFiles = 0;

		private struct JobProperties 
		{
			public string Ticket;
			public string LastState;
			public string CompilerOutput;
			public string Code;

			public JobProperties (string ticket)
			{
				this.Ticket = ticket;
				this.LastState = null;
				this.CompilerOutput = null;
				this.Code = null;
			}
		}
        
		private ImageList imageList;
		private System.Windows.Forms.ImageList menuImageList;
		private PopupMenu contextMenu;
		private MenuCommand mcCancel;
		private MenuCommand mcDelete;
		private MenuCommand mcViewResults;
		private MenuCommand mcViewCode;
		private MenuCommand mcStartOrStopUpdating;
		private System.Windows.Forms.StatusBar statusBar;
		private System.Windows.Forms.StatusBarPanel sbPanelText;
		private IContainer components;

		// Delegates
		private delegate int AddTreeNodeBottomDelegate(TreeNode node);
		private delegate void AddTreeNodeTopDelegate (int index, TreeNode node);
		private delegate void CommandDelegate(TreeNode node);
        
		public DCL_VSI DCL_VSI
		{
			set {dclvsi = value;}
		}
        
		public Mutex JobExplorerLock
		{
			get {return jobExplorerLock;}
		}
        
		public string StatusBarMsg
		{
			set {sbPanelText.Text = value;}
		}

        
		public void setServer(Settings s)
		{
			ticketServer = s.TicketServer;
			experimentService = s.ExperimentService;
			settings = s;
		}

		public bool IsRefreshing
		{
			get {return isRefreshing;}
		}

		public JobExplorer()
		{        
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
			
			// Initialize Job Explorer's context menu
			InitializeOwnComponents();

			jobExplorerLock = new Mutex();
            
			usedExperiments = new Hashtable();

			// Font for tree nodes, represents jobs
			jobFont = new Font("Microsoft Sans Serif", 8.25f, FontStyle.Regular);

			convertToJobStatusType = new Hashtable();
			convertToJobStatusType.Add("queued", JobStatusType.Queued);
			convertToJobStatusType.Add("compilation", JobStatusType.Compilation);
			convertToJobStatusType.Add("execution", JobStatusType.Execution);
			convertToJobStatusType.Add("finished_Failed", JobStatusType.Failed);
			convertToJobStatusType.Add("finished_Failed Experiment Down", JobStatusType.Failed);
			convertToJobStatusType.Add("finished_Stop Call Failed", JobStatusType.Failed);
			convertToJobStatusType.Add("Finish_Experiment crashed", JobStatusType.Failed);
			convertToJobStatusType.Add("canceled", JobStatusType.Canceled);
			convertToJobStatusType.Add("finished_Successful", JobStatusType.Finished);
			convertToJobStatusType.Add("finished_StopSignal", JobStatusType.Finished);
			convertToJobStatusType.Add("finished_Timeover", JobStatusType.Finished);

			jobStatusCaption = new Hashtable();
			jobStatusCaption.Add(JobStatusType.Unknown, "Unknown");
			jobStatusCaption.Add(JobStatusType.Queued, "Queued");
			jobStatusCaption.Add(JobStatusType.Compilation, "Compilation");
			jobStatusCaption.Add(JobStatusType.Execution, "Execution");
			jobStatusCaption.Add(JobStatusType.Failed, "Failed");
			jobStatusCaption.Add(JobStatusType.Canceled, "Canceled");
			jobStatusCaption.Add(JobStatusType.Finished, "Finished");
            
			resultCode = new Hashtable();
			resultCode.Add("execution", "Currently executing.");
			resultCode.Add("canceled", "Canceled");
			resultCode.Add("finished_Failed", "Failed");
			resultCode.Add("finished_Failed Experiment Down", "Experiment down.");
			resultCode.Add("finished_Stop Call Failed", "Stop Call failed.");
			resultCode.Add("Finish_Experiment crashed", "Experiment crashed.");
			resultCode.Add("finished_Successful", "Successful");
			resultCode.Add("finished_StopSignal", "Stop signal.");
			resultCode.Add("finished_Timeover", "Time over.");


		}

		public void reset()
		{
			stopUpdateJobStatusThread();
            
			jobExplorerLock.WaitOne();
			
			// Delete collection of used experiments currently loaded in 'Job Explorer'
			usedExperiments.Clear();
            
			// Delete all Jobs currently loaded in 'Job Explorer'
			jobs.Nodes.Clear();			
            
			jobExplorerLock.ReleaseMutex();

			isRefreshing = false;
            
			System.Threading.Thread.CurrentThread.Abort();
		}

		public void clear()
		{
			// Delete collection of used experiments currently loaded in 'Job Explorer'
			usedExperiments.Clear();
            
			// Delete all Jobs currently loaded in 'Job Exlorer'
			jobs.Nodes.Clear();
		}
        
		public void refreshJobs()
		{    
			jobExplorerLock.WaitOne();
            
			mcStartOrStopUpdating.Visible = false;

			isRefreshing = true;
            
			// Delete all Jobs currently loaded in 'Job Exlorer'
			clear();

			jobExplorerLock.ReleaseMutex();

			// Check if connection to DCL is available
			if (settings.checkConnection()==null)
			{            
				try
				{
					// Get all tickets of the user
					string[] ticketList = null;
					bool viewAll;
					ticketList = ticketServer.GetTicketList(settings.Username, settings.Password, out viewAll);
                
					if (ticketList[0]=="S_ERROR_NOTHING_FOUND")
					{                    
						sbPanelText.Text = "No experiments found";
						isRefreshing = false;
						return;
					}
					if (ticketList[0]=="S_ERROR_INVALID_LOGIN")
					{
						Error.notify("Refresh failed. Invalid username or password.");
						sbPanelText.Text = "Refresh failed";
						reset();
					}
                
					sbPanelText.Text = "Loading ...";

					foreach (string ticket in ticketList)
					{
						jobExplorerLock.WaitOne();

						// Add a job in 'Job Explorer'
						addJob(ticket, false);
                        
						jobExplorerLock.ReleaseMutex();
					}
				}
				catch (ThreadAbortException){}
				catch (SoapException e)
				{
					Error.handleSoapException("Refresh of Job Explorer failed.",e);
					sbPanelText.Text = "Refresh failed";
					reset();
				}
				catch (Exception e)
				{
					Error.handleConnectionException("Refresh of Job Explorer failed.", e);
					sbPanelText.Text = "Refresh failed";                    
					reset();
				}                
			}    
            
			startUpdateJobStatusThread();
			mcStartOrStopUpdating.Visible = true;

			isRefreshing = false;
			
			sbPanelText.Text = "Ready";
		}        
        
		public TreeNode addJob(string ticket, bool top)
		{
			string experiment = null;
			TreeNode tnResult = null;
			// determine the experiment type of the job            
			experiment = experimentService.GetExperimentTypeofTicket(ticket);
			if (experiment.StartsWith("S_ERROR") == true)
			{
				// delete ticket
				try
				{
					ticketServer.FreeTicket(ticket);
					experimentService.FreeTicket(ticket);
				}
				catch {}
			}
			else
			{
				// Create new experiment tree node, if not exists
				if (usedExperiments.ContainsKey(experiment) == false)
				{
					// determine tree node index for the new experiment
					int experimentIndex = usedExperiments.Count;
					usedExperiments.Add(experiment, experimentIndex);
                
					// Create new tree node, which represents experiment
					TreeNode tn = new TreeNode(experiment, experimentImage, experimentImage);
					tn.Tag = experiment;
                
					// Add the new experiment to 'Job Explorer'
					AddTreeNodeBottomDelegate addTreeNode = new AddTreeNodeBottomDelegate(jobs.Nodes.Add);
					object[] args = new object[]{tn};
					jobs.Invoke(addTreeNode, args);

					// Updates the caption of the node
					updateExperimentNode(experiment);
				}
                
				// Create new job tree node
				tnResult = addJobNode(ticket, experiment, top);
			}
			return tnResult;
		}        
        
		private void updateExperimentNode(string experiment)
		{
			// Create caption of experiment tree node
			int queueOfExperiment = -1;
			queueOfExperiment = experimentService.GetQueueofExperiment(experiment);
            
			string experimentCaption = null;
			if (queueOfExperiment != -1)
			{
				experimentCaption = experiment + " (" + queueOfExperiment.ToString() + ")";
			}
			else
			{
				experimentCaption = experiment;
			}

			// Update experiment in 'Job Exlorer'
			jobs.Nodes[(int)usedExperiments[experiment]].Text = experimentCaption;                            
		}

		private TreeNode addJobNode(string ticket, string experiment, bool top)
		{
			string jobCaption = null;
            
			// determine the status of the job                
			string status = experimentService.GetExperimentStatus(ticket);
			if (status.StartsWith("S_ERROR")==true)
			{
				// delete ticket
				try
				{
					ticketServer.FreeTicket(ticket);
					experimentService.FreeTicket(ticket);
				}
				catch {}
				return null;
			}
			else
			{
				// Determine start time
				string startTime;
				try
				{
					startTime = ticketServer.GetUsageStart(ticket);
					DateTime dtStartTime = Convert.ToDateTime(startTime);
					startTime = dtStartTime.ToString("dd\\.MM\\.yy HH\\:mm \\U\\h\\r");
				}
				catch
				{
					startTime = "Unknown";
				}
                
				// Convert job status to intern representation
				JobStatusType jobStatus = JobStatusType.Unknown;
				if (convertToJobStatusType.ContainsKey(status) == true)
				{
					jobStatus = (JobStatusType) convertToJobStatusType[status];
				}
				/* Only for debugging
				else
				{
					Error.log("JobExlorer", "addJobNode()", "Experiment: " + experiment + "\nStatus \"" + status + "\" not supported.");
				}
				*/
                
				// If job is queued, add number of jobs to caption, which are executed before
				if (jobStatus == JobStatusType.Queued)
				{
					int queueTime = experimentService.GetQueueTime(ticket);
					jobCaption = startTime + " (" + jobStatusCaption[jobStatus].ToString() + ": " + queueTime + ")";
				}
				else
				{
					jobCaption = startTime + " (" + jobStatusCaption[jobStatus].ToString() + ")";
				}

				// Create new TreeNode for the job
				TreeNode tn = new TreeNode(jobCaption, (int)jobStatus, (int)jobStatus);
				JobProperties jobProp = new JobProperties(ticket);
				jobProp.LastState = status;
				tn.Tag = jobProp;
				tn.NodeFont = jobFont;
                
				// Add job tree node in 'Job Explorer'
				if (top == true)
				{
					AddTreeNodeTopDelegate addTreeNode = new AddTreeNodeTopDelegate(
						jobs.Nodes[(int)usedExperiments[experiment]].Nodes.Insert);
					object[] args = new object[]{0, tn};
					jobs.Invoke(addTreeNode, args);
				}
				else
				{
					AddTreeNodeBottomDelegate addTreeNode = new AddTreeNodeBottomDelegate(
						jobs.Nodes[(int)usedExperiments[experiment]].Nodes.Add);
					object[] args = new object[]{tn};
					jobs.Invoke(addTreeNode, args);
				}
				return tn;
			}            
		}

		private void startUpdateJobStatusThread()
		{
			// Start update thread
			try
			{
				if (updateJobStatusThread != null)
				{
					updateJobStatusThread.Abort();
				}
				updateJobStatusThread = new System.Threading.Thread(new ThreadStart(updateJobs));
				updateJobStatusThread.Start();

				// Change caption of context menu item
				mcStartOrStopUpdating.Description = "Stop Updating Jobs";
				mcStartOrStopUpdating.Text = "&Stop Updating";
			}
			catch (ThreadAbortException) {}            
		}
		
		private void stopUpdateJobStatusThread()
		{
			// Stop update thread
			if (updateJobStatusThread != null)
			{
				updateJobStatusThread.Abort();
				updateJobStatusThread = null;
				
				// Change caption of context menu item
				mcStartOrStopUpdating.Description = "Start Updating Jobs";
				mcStartOrStopUpdating.Text = "&Start Updating";
			}
		}


		private void updateJobs()
		{
			int checkForNewOrDeletedJobsCounter = 0;

			while (true)
			{
				// Get Mutex
				jobExplorerLock.WaitOne();

				if (checkForNewOrDeletedJobsCounter == 3)
				{
					checkForNewOrDeletedJobs();
				}
                
				try
				{                
					foreach (TreeNode experimentTreeNode in jobs.Nodes)
					{
						if(experimentTreeNode.IsExpanded == true)
						{
							int i = 0;
							while ( ((experimentTreeNode.Nodes[i].ImageIndex < (int)JobStatusType.Failed)
								      ||(experimentTreeNode.Nodes[i].ImageIndex == (int)JobStatusType.Canceled))
								     &&(i < experimentTreeNode.Nodes.Count)) 
							{
								if (experimentTreeNode.Nodes[i].ImageIndex != (int)JobStatusType.Canceled)
								{
									updateJobNode(experimentTreeNode.Nodes[i]);
								}
								i++;
							}                            
						}
						updateExperimentNode((string)experimentTreeNode.Tag);
					}
				}
				catch {}
				finally
				{                
					jobExplorerLock.ReleaseMutex();
					checkForNewOrDeletedJobsCounter = (checkForNewOrDeletedJobsCounter + 1) % 4;
					System.Threading.Thread.Sleep(settings.PollingRate);
				}            
			}
		}

		private void updateJobNode(TreeNode tn)
		{			
			JobProperties jobProp = (JobProperties)tn.Tag;            
			// Determine the status of the job
			string status = experimentService.GetExperimentStatus(jobProp.Ticket);

			if (status.StartsWith("S_ERROR")==true)
			{
				tn.ImageIndex= (int)JobStatusType.Unknown;
				tn.SelectedImageIndex = (int)JobStatusType.Unknown;
			}
			else
			{
				jobProp.LastState = status;
				tn.Tag = jobProp;

				int endIndexOfStartTime = tn.Text.LastIndexOf('(') - 1;
				string startTime = tn.Text.Substring(0, endIndexOfStartTime);
                
				JobStatusType jobStatus = JobStatusType.Unknown;
				if (convertToJobStatusType.ContainsKey(status) == true)
				{
					jobStatus = (JobStatusType) convertToJobStatusType[status];
				}
                
				string jobCaption;
				if (jobStatus == JobStatusType.Queued)
				{
					int queueTime = experimentService.GetQueueTime(jobProp.Ticket);
					jobCaption = startTime + " (" + jobStatusCaption[jobStatus].ToString() + ": " + queueTime + ")";
				}
				else
				{
					jobCaption = startTime + " (" + jobStatusCaption[jobStatus].ToString() + ")";
				}
                
				tn.Text = jobCaption;
				tn.ImageIndex = (int) jobStatus;
				tn.SelectedImageIndex = (int)jobStatus;
			}
		}

		private void checkForNewOrDeletedJobs()
		{            
			// Get all tickets of the user
			ArrayList ticketList = null;            
			bool viewAll;
			ticketList = new ArrayList(ticketServer.GetTicketList(settings.Username, settings.Password, out viewAll));
                
			if ((string)ticketList[0]=="S_ERROR_NOTHING_FOUND")
			{            
				return;
			}
			if ((string)ticketList[0]=="S_ERROR_INVALID_LOGIN")
			{
				return;
			}
            
			int i = 0;            
			while (i < jobs.Nodes.Count)
			{
				int j = 0;
				while (j < jobs.Nodes[i].Nodes.Count) 
				{
					// If job is not in ticket list, remove them from 'Job Explorer'
					int indexOfTicket = ticketList.IndexOf(((JobProperties)jobs.Nodes[i].Nodes[j].Tag).Ticket);
					if ( indexOfTicket == -1)
					{
						jobs.Nodes[i].Nodes[j].Remove();
						jobs.Nodes[i].Nodes[j] = null;
					}
					// If job is in ticket list, remove them from ticket list
					else
					{
						ticketList.RemoveAt(indexOfTicket);
						j++;
					}                    
				}
                
				// Remove experiment node, if node has no jobs
				if(jobs.Nodes[i].Nodes.Count == 0)
				{
					usedExperiments.Remove((string)jobs.Nodes[i].Tag);
					jobs.Nodes[i].Remove();
				}
				else
				{
					i++;
				}
			}
            
			// Add each job in ticket list to 'Job Explorer'
			i = 0;
			while (i < ticketList.Count)
			{
				addJob((string)ticketList[i], true);
				i++;
			}
		}
        
		public void deleteJobNode(TreeNode tn)
		{
			// If experiment have no jobs, then delete experiment node
			if (tn.Parent.Nodes.Count == 1)
			{
				usedExperiments.Remove((string)tn.Parent.Tag);
				tn.Parent.Remove();
				tn = null;
			}
			else
			{
				tn.Remove();
				tn = null;
			}
		}
        
		public void selectJobNode(TreeNode tn)
		{
			jobs.SelectedNode = tn;
		}
		
		private void jobs_OnMouseUp(object sender, MouseEventArgs e)
		{            
			if (e.Button == MouseButtons.Right)
			{
				TreeNode selectedNode = jobs.GetNodeAt(e.X, e.Y);
				
				if (selectedNode != null)
				{
					// Select job node
					selectJobNode(selectedNode);

					if (selectedNode.ImageIndex != experimentImage)
					{                
						// Context menu for unknown jobs
						if (selectedNode.ImageIndex == (int)JobStatusType.Unknown)
						{
							mcCancel.Visible = false;
							mcDelete.Visible = true;
							mcViewCode.Visible = true;
							mcViewResults.Visible = true;                
						}
						// Context menu for queued jobs
						else if (selectedNode.ImageIndex == (int)JobStatusType.Queued)
						{
							mcCancel.Visible = true;
							mcDelete.Visible = false;
							mcViewCode.Visible = true;
							mcViewResults.Visible = false;
						}
						// Context menu for compiling jobs
						else if (selectedNode.ImageIndex == (int)JobStatusType.Compilation)
						{
							mcCancel.Visible = false;
							mcDelete.Visible = false;
							mcViewCode.Visible = true;
							mcViewResults.Visible = false;
						}
						// Context menu for executing jobs
						else if (selectedNode.ImageIndex == (int)JobStatusType.Execution)
						{
							mcCancel.Visible = true;
							mcDelete.Visible = false;
							mcViewCode.Visible = true;
							mcViewResults.Visible = false;
						}
						// Context menu for jobs in a final state
						else if ( ((int)JobStatusType.Execution < selectedNode.ImageIndex) 
							&& (selectedNode.ImageIndex < experimentImage) )
						{
							mcCancel.Visible = false;
							mcDelete.Visible = true;
							mcViewCode.Visible = true;
							mcViewResults.Visible = true;
						}
						// Show context menu
						contextMenu.TrackPopup(jobs.PointToScreen(new Point(e.X, e.Y)));
					}
				}
				else if (jobs.Nodes.Count != 0)
				{
					mcCancel.Visible = false;
					mcDelete.Visible = false;
					mcViewCode.Visible = false;
					mcViewResults.Visible = false;
					// Show context menu
					contextMenu.TrackPopup(jobs.PointToScreen(new Point(e.X, e.Y)));
				}
			}
		}

		private void jobs_OnDoubleClick(object sender, System.EventArgs e)
		{
			TreeNode selectedNode = jobs.SelectedNode;		

			if (selectedNode.ImageIndex != experimentImage)
			{
				// Start thread to open a document within the code of the selected job
				CommandArgs args = new CommandArgs(jobs.SelectedNode);            
				args.Command = new CommandDelegate(this.viewCodeThread);
				try
				{
					System.Threading.Thread t = new System.Threading.Thread (new ThreadStart(args.ExecCommand));
					t.Start();
				}
				catch (ThreadAbortException){}

				viewCompilerOutput(selectedNode);
			}		
		}
        
		private class CommandArgs 
		{
			private TreeNode treeNode;
			public CommandDelegate Command;

			public CommandArgs(TreeNode tn)
			{
				this.treeNode = tn;
			}            
			public void ExecCommand() 
			{                
				Command(treeNode);
			}
		}
        
		private void mcCancel_Click(object sender, System.EventArgs e)
		{
			CommandArgs args = new CommandArgs(jobs.SelectedNode);            
			args.Command = new CommandDelegate(this.cancelThread);
			try
			{
				System.Threading.Thread t = new System.Threading.Thread (new ThreadStart(args.ExecCommand));
				t.Start();
			}
			catch (ThreadAbortException){}    
		}

		private void cancelThread(TreeNode tn)
		{
			// Get mutex CommandLock
			jobExplorerLock.WaitOne();

			try 
			{
				string result;
				result = experimentService.CancelExperiment (((JobProperties)tn.Tag).Ticket);
				
				if (result != "S_OK")
				{
					Error.notify("Cancel Job failed.");
				}
			}
			catch (SoapException e)
			{
				Error.handleSoapException("Cancel job failed.",e);
				System.Threading.Thread.CurrentThread.Abort();
			}
			catch (Exception e)
			{
				Error.handleConnectionException("Cancel job failed.", e);
				System.Threading.Thread.CurrentThread.Abort();
			}
			finally 
			{
				updateJobNode(tn);
				jobExplorerLock.ReleaseMutex();
			}
		}
        
        
		private void mcDelete_Click(object sender, System.EventArgs e)
		{
			// Get mutex
			jobExplorerLock.WaitOne();
			CommandArgs args = new CommandArgs(jobs.SelectedNode);
			// Release mutex
			jobExplorerLock.ReleaseMutex();
			args.Command = new CommandDelegate(this.deleteThread);
			try
			{
				System.Threading.Thread t = new System.Threading.Thread (new ThreadStart(args.ExecCommand));
				t.Start();
			}
			catch (ThreadAbortException){}
		}

		private void deleteThread(TreeNode tn)
		{
			// Get mutex CommandLock
			jobExplorerLock.WaitOne();

			try 
			{
				ticketServer.FreeTicket(((JobProperties)tn.Tag).Ticket);
				experimentService.FreeTicket(((JobProperties)tn.Tag).Ticket);
                
				deleteJobNode(tn);
			}
			catch (SoapException e)
			{
				Error.handleSoapException("Delete job failed.",e);
				System.Threading.Thread.CurrentThread.Abort();
			}
			catch (Exception e)
			{
				Error.handleConnectionException("Delete job failed.", e);
				System.Threading.Thread.CurrentThread.Abort();
			}
			finally 
			{
				jobExplorerLock.ReleaseMutex();
			}
		}

    
		private void mcViewCode_Click(object sender, System.EventArgs e)
		{
			CommandArgs args = new CommandArgs(jobs.SelectedNode);            
			args.Command = new CommandDelegate(this.viewCodeThread);
			try
			{
				System.Threading.Thread t = new System.Threading.Thread (new ThreadStart(args.ExecCommand));
				t.Start();
			}
			catch (ThreadAbortException){}
		}

		private void viewCodeThread(TreeNode tn)
		{
			// Get mutex CommandLock
			jobExplorerLock.WaitOne();
            
			try 
			{
				JobProperties jobProp = (JobProperties)tn.Tag;
                
				string code = jobProp.Code;                
				if (code == null)
				{
					// Get code from DCL
					code = experimentService.GetStringResult(jobProp.Ticket, "Code");                
				}
                
				if (code != "S_ERROR_UNKNOWN_TICKET")
				{
					jobProp.Code = code;
					tn.Tag = jobProp;                    
					// Create new textfile
					openCodeFiles++;
					string filename = (string)tn.Parent.Tag + " " + openCodeFiles.ToString() + ".txt";
					dclvsi.openNewCodeWindow (filename, code);
				}
				else 
				{
					Error.notify("Unable to get code from DCL: " + (string)tn.Parent.Tag + " - " + tn.Text);
				}

			}
			catch (SoapException e)
			{
				Error.handleSoapException("View code failed.",e);
				System.Threading.Thread.CurrentThread.Abort();
			}
			catch (Exception e)
			{
				Error.handleConnectionException("View code failed.", e);
				System.Threading.Thread.CurrentThread.Abort();
			}
			finally 
			{
				jobExplorerLock.ReleaseMutex();
			}
		}


		private void mcViewResults_Click(object sender, System.EventArgs e)
		{
			TreeNode tn = jobs.SelectedNode;
            
			// Show results in 'Output' window
			viewCompilerOutput(tn);

			// Show result web page
			string url = settings.ResultUrlHeader + ((JobProperties) tn.Tag).Ticket;
			dclvsi.showInternetExplorer(url);
		}

		public void viewCompilerOutput(TreeNode tn)
		{
			// Abort running thread
			try
			{
				compilerOutputThread.Abort();
			}
			catch{}
            
			// Start new thread
			CommandArgs args = new CommandArgs(tn);            
			args.Command = new CommandDelegate(this.viewCompilerOutputThread);
			try
			{
				compilerOutputThread = new System.Threading.Thread (new ThreadStart(args.ExecCommand));
				compilerOutputThread.Start();
			}
			catch (ThreadAbortException){}
		}

		private void viewCompilerOutputThread(TreeNode tn)
		{            
			jobExplorerLock.WaitOne();
            
			if (tn == null) 
			{
				jobExplorerLock.ReleaseMutex();
				System.Threading.Thread.CurrentThread.Abort();
			}
            
			JobProperties jobProp = (JobProperties)tn.Tag;
            
			dclvsi.printToOutputWindow("Experiment: " + (string)tn.Parent.Tag + "\n", false);
            
			int endIndexOfStartTime = tn.Text.LastIndexOf('(') - 1;
			string startTime = tn.Text.Substring(0, endIndexOfStartTime);            
			dclvsi.printToOutputWindow("Job's start time: " + startTime + "\n\n", true);

			jobExplorerLock.ReleaseMutex();
            
			bool printedCompiling = false;
			bool printedCompiled = false;
			bool printedExecuting = false;
            
			while (true)
			{
				// Get mutex CommandLock
				jobExplorerLock.WaitOne();

				if (tn == null) 
				{
					jobExplorerLock.ReleaseMutex();
					System.Threading.Thread.CurrentThread.Abort();
				}

				try 
				{
					jobProp = (JobProperties)tn.Tag;

					if ( (tn.ImageIndex == (int)JobStatusType.Compilation) && (printedCompiling == false) )
					{
						dclvsi.printToOutputWindow("Compiling...\n", true);
						printedCompiling = true;                        
					}

					if ( (tn.ImageIndex > (int)JobStatusType.Compilation) && (printedCompiling == true) )
					{
						if (printedCompiled == false)
						{
							dclvsi.printToOutputWindow("Compiling finished.\n\n", true);
							printedCompiled = true;
						}
					}

					if ( (tn.ImageIndex == (int)JobStatusType.Execution) && (printedExecuting == false) )
					{
						dclvsi.printToOutputWindow("Executing...\n", true);
						printedExecuting = true;
					}

					// Job is in a final state
					if (tn.ImageIndex > (int)JobStatusType.Execution)
					{
						if (printedExecuting == true)
						{
							dclvsi.printToOutputWindow("Execution finished.\n\n", true);
						}

						// Job finished in state "failed" - compiler-output is printed
						if (tn.ImageIndex == (int)JobStatusType.Failed)
						{
							dclvsi.printToOutputWindow("Getting Compiler Output ...\n", true);

							if (jobProp.CompilerOutput == null)
							{                
								string[] resultList = null;
								// Get list of results
								resultList = experimentService.GetResultList(jobProp.Ticket);
                                
								// Get the output from the compiler
								if (resultList != null)
								{
									foreach (string result in resultList)
									{
										string type;
										type = experimentService.GetResultType(result.Trim());

										if (result.Trim() == "Compiler Output")
										{                                
											jobProp.CompilerOutput = 
												experimentService.GetStringResult(jobProp.Ticket, "Compiler Output");
											tn.Tag = jobProp;
										}
									}
								}
							}
							if (jobProp.CompilerOutput != null)
							{
								dclvsi.printToOutputWindow(jobProp.CompilerOutput + "\n", true);
							}
							else
							{
								dclvsi.printToOutputWindow("No compiler output available.\n", true);
							}
						}

						// Add final state to output text
						if (resultCode.ContainsKey(jobProp.LastState))
						{
							dclvsi.printToOutputWindow("\nResult Code: " + resultCode[jobProp.LastState].ToString(), true);
						}
						else
						{
							dclvsi.printToOutputWindow("\nResult Code: " + "Unknown", true);
						}

						jobExplorerLock.ReleaseMutex();
						System.Threading.Thread.CurrentThread.Abort();
					}
				}
				catch (ThreadAbortException){}
				catch (SoapException e)
				{
					Error.handleSoapException("View results failed.",e);
					System.Threading.Thread.CurrentThread.Abort();
				}
				catch (Exception e)
				{
					Error.handleConnectionException("View results failed.", e);
					System.Threading.Thread.CurrentThread.Abort();
				}
				finally 
				{
					jobExplorerLock.ReleaseMutex();
					System.Threading.Thread.Sleep(settings.PollingRate);
				}
			}
		}

		private void mcStartOrStopUpdating_Click(object sender, System.EventArgs e)
		{
			if (updateJobStatusThread != null)
			{
				stopUpdateJobStatusThread();
			}
			else
			{
				startUpdateJobStatusThread();
			}
		}

		// Clean up any resources being used
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(JobExplorer));
			this.jobs = new System.Windows.Forms.TreeView();
			this.imageList = new System.Windows.Forms.ImageList(this.components);
			this.menuImageList = new System.Windows.Forms.ImageList(this.components);
			this.statusBar = new System.Windows.Forms.StatusBar();
			this.sbPanelText = new System.Windows.Forms.StatusBarPanel();
			((System.ComponentModel.ISupportInitialize)(this.sbPanelText)).BeginInit();
			this.SuspendLayout();
			// 
			// jobs
			// 
			this.jobs.Dock = System.Windows.Forms.DockStyle.Fill;
			this.jobs.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.jobs.ImageList = this.imageList;
			this.jobs.Location = new System.Drawing.Point(0, 0);
			this.jobs.Name = "jobs";
			this.jobs.Size = new System.Drawing.Size(150, 112);
			this.jobs.TabIndex = 0;
			this.jobs.MouseUp += new System.Windows.Forms.MouseEventHandler(this.jobs_OnMouseUp);
			this.jobs.DoubleClick += new System.EventHandler(this.jobs_OnDoubleClick);
			// 
			// imageList
			// 
			this.imageList.ImageSize = new System.Drawing.Size(16, 16);
			this.imageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList.ImageStream")));
			this.imageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// menuImageList
			// 
			this.menuImageList.ImageSize = new System.Drawing.Size(16, 16);
			this.menuImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("menuImageList.ImageStream")));
			this.menuImageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// statusBar
			// 
			this.statusBar.Location = new System.Drawing.Point(0, 88);
			this.statusBar.Name = "statusBar";
			this.statusBar.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																						 this.sbPanelText});
			this.statusBar.ShowPanels = true;
			this.statusBar.Size = new System.Drawing.Size(150, 24);
			this.statusBar.SizingGrip = false;
			this.statusBar.TabIndex = 1;
			// 
			// sbPanelText
			// 
			this.sbPanelText.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.sbPanelText.Width = 150;
			// 
			// JobExplorer
			// 
			this.Controls.Add(this.statusBar);
			this.Controls.Add(this.jobs);
			this.Name = "JobExplorer";
			this.Size = new System.Drawing.Size(150, 112);
			((System.ComponentModel.ISupportInitialize)(this.sbPanelText)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		private void InitializeOwnComponents()
		{
			this.mcCancel = new Crownwood.Magic.Menus.MenuCommand();
			this.mcDelete = new Crownwood.Magic.Menus.MenuCommand();
			this.mcViewResults = new Crownwood.Magic.Menus.MenuCommand();
			this.mcViewCode = new Crownwood.Magic.Menus.MenuCommand();
			this.mcStartOrStopUpdating = new Crownwood.Magic.Menus.MenuCommand();
			
			this.contextMenu = new Crownwood.Magic.Menus.PopupMenu();
			
			// mcCancel
			this.mcCancel.Description = "Cancel Job";
			this.mcCancel.ImageIndex = 0;
			this.mcCancel.ImageList = this.menuImageList;
			this.mcCancel.Text = "&Cancel Job";
			this.mcCancel.Click += new EventHandler(mcCancel_Click);

			// mcDelete
			this.mcDelete.Description = "Delete Job";
			this.mcDelete.ImageIndex = 1;
			this.mcDelete.ImageList = this.menuImageList;
			this.mcDelete.Text = "&Delete Job";
			this.mcDelete.Click += new EventHandler(mcDelete_Click);

			// mcViewCode
			this.mcViewCode.Description = "View Code";
			this.mcViewCode.ImageIndex = 2;
			this.mcViewCode.ImageList = this.menuImageList;
			this.mcViewCode.Text = "View &Code";
			this.mcViewCode.Click += new EventHandler(mcViewCode_Click);

			// mcViewResults
			this.mcViewResults.Description = "View Results";
			this.mcViewResults.ImageIndex = 3;
			this.mcViewResults.ImageList = this.menuImageList;
			this.mcViewResults.Text = "View &Results";
			this.mcViewResults.Click += new EventHandler(mcViewResults_Click);

			// mcStartOrStopUpdating			
			//this.StartOrStopUpdating.ImageIndex = 3;
			//this.StartOrStopUpdating.ImageList = this.menuImageList;			
			this.mcStartOrStopUpdating.Click += new EventHandler(mcStartOrStopUpdating_Click);

			// contextMenu
			this.contextMenu.MenuCommands.AddRange(new Crownwood.Magic.Menus.MenuCommand[]{
																							this.mcCancel,
																							this.mcDelete,
																							this.mcViewCode,
																							this.mcViewResults,
																							this.mcStartOrStopUpdating});                
		}

	}
}

