using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Reflection;
using System.Xml;
using Microsoft.Win32;
using DCL_VSI.de.discourse.was;
using DCL_VSI.de.discourse.was1;
using System.Threading;


namespace DCL_VSI
{


	public class Settings : System.Windows.Forms.Form
	{
		private string regkey = null;
		private string username = null;
		private string password = null;
		
		private string configFileName = null;
		private string ticketServerUrl = null;
		private string experimentServiceUrl = null;
		private string resultUrlHeader = null;
		private int pollingRate = 3000;
		private ArrayList cameraNames = null;
		private ArrayList cameraUrls = null;

		private string startupErrors = null;
		private Thread testThread;

		private DCL_VSI dclvsi;
		private DCLTicketServer ticketServer;
		private DCLExperimentService experimentService;

		private System.Windows.Forms.Button btmTest;
		private System.Windows.Forms.Button btmOk;
		private System.Windows.Forms.Button btmCancel;
		private System.Windows.Forms.TextBox txtUsername;
		private System.Windows.Forms.TextBox txtPassword;
		private System.Windows.Forms.Label lblUsername;
		private System.Windows.Forms.Label lblPassword;
		private System.Windows.Forms.TextBox txtError;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.GroupBox groupBox2;
		
		// Required designer variable.
		private System.ComponentModel.Container components = null;

		
		public string Username
		{
			get {return username; }
		}

		public string Password
		{
			get {return password; }
		}

		public DCLTicketServer TicketServer
		{
			get {return ticketServer; }
		}

		public DCLExperimentService ExperimentService
		{
			get {return experimentService; }
		}

		public string ResultUrlHeader
		{
			get {return resultUrlHeader; }
		}

		public int PollingRate
		{
			get {return pollingRate; }
		}

		public ArrayList CameraNames
		{
			get	{return cameraNames;}
		}
		
		public ArrayList CameraUrls
		{
			get {return cameraUrls;}
		}				
		

		public Settings()
		{
			// Required for Windows Form Designer support
			InitializeComponent();
		}

		public new void ShowDialog()
		{
			this.btmTest_Click(null, null);
			base.ShowDialog();
		}

		
		public void startup(DCL_VSI dv, string registryRoot)
		{
			dclvsi = dv;
			startupErrors = "";
			
			// set registrykey
			regkey = registryRoot + "\\AddIns\\DCL_VSI.connect";
			
			// read username and password from registry
			readRegistry();
			txtUsername.Text = username;
			txtPassword.Text = password;

			// read config.xlm
			ticketServerUrl = "";
			experimentServiceUrl = "";
			resultUrlHeader = "";
			cameraNames = new ArrayList();
			cameraUrls = new ArrayList();
			setFileName();
			readConfigFile();

			// initialize DCL-services
			try
			{
				this.ticketServer = new DCLTicketServer();
				this.experimentService = new DCLExperimentService();
				if (ticketServerUrl != "") 
				{
					this.ticketServer.Url = ticketServerUrl;
				}
				if (experimentServiceUrl != "") 
				{
					this.experimentService.Url = experimentServiceUrl;
				}
			}
			catch (Exception e)
			{
				Error.notify("Unknown Error." + Error.FatalErrorInfo);
				Error.log (e);
			}

			if (startupErrors != "")
			{
				startupErrors = "Startup:\r\n" + startupErrors;
				txtError.Text = startupErrors;
			}
		}


		private bool readRegistry() 
		{
			string usernameRK = null;
			string passwordRK = null;
			bool error = false;
			
			try
			{
				RegistryKey key = Registry.CurrentUser;
				key = key.OpenSubKey(regkey);
				if (key == null)
				{
					// DCL VSI key not found in registry
					error = true;
					Error.log ("Settings", "readRegistry()", "Unable to find key: \"" + regkey + "\" (OpenSubKey()).");
				}
				
				if (error == false)
				{
					usernameRK = (string) key.GetValue("username");
					passwordRK = (string) key.GetValue("password");
					if (usernameRK == null | passwordRK == null)
					{
						// unable to find username or password in registry
						error = true;
						Error.log ("Settings", "readRegistry()", "Unable to find username or password (GetValue()).");
					}
				}
			}
			catch (System.Exception e) 
			{
				error = true;
				Error.log (e);
			}

			if (error == false)
			{
				this.username = usernameRK;
				this.password = passwordRK;
				return true;
			}
			else 
			{
				startupErrors = startupErrors + "Unable to read username and password from registry.\r\n";
				return false;
			}
		}


		private bool writeRegistry() 
		{
			bool error = false;
			
			try
			{
				RegistryKey key = Registry.CurrentUser;
				key = key.OpenSubKey(regkey, true);
				if (key == null) 
				{
					// DCL VSI key not found in registry
					error = true;
					Error.notify("Unable to write username and password to registry.");
					Error.log ("Settings", "writeRegistry()", "Unable to find key: \"" + regkey + "\" (OpenSubKey()).");
					
				}
				if (error == false)
				{
					// write values to the registry
					key.SetValue("username", username);
					key.SetValue("password", password);
				}
			}
			catch (System.Exception e) 
			{
				error = true;
				Error.notify("Unable to write username and password to registry.");
				Error.log (e);
			}

			if (error == false) return true;
			else return false;

		}

		
		private void setFileName()
		{
			// set path and filename of the installed config.xml
			Assembly asm = Assembly.GetExecutingAssembly();	
			configFileName = asm.Location.Substring(0,asm.Location.LastIndexOf('\\') + 1) + "config.xml";
		}

		
		private void readConfigFile() 
		{
			XmlDocument xmlDoc = new XmlDocument();

			try
			{
				// Try to load config.xml
				xmlDoc.Load(configFileName);
				XmlNode xmlNode;

				// Get ticketServerUrl from config.xml
				try
				{
					xmlNode = xmlDoc.SelectSingleNode("settings/ticketServerUrl"); 
					ticketServerUrl = xmlNode.InnerText;
				}
				catch (Exception e)
				{
					startupErrors = startupErrors + "Unable to to find ticketServerUrl-node in config.xml.\r\n";
					Error.log(e);
				}

				// Get experimentServiceUrl from config.xml
				try
				{
					xmlNode = xmlDoc.SelectSingleNode("settings/experimentServiceUrl"); 
					experimentServiceUrl = xmlNode.InnerText;
				}
				catch (Exception e)
				{
					startupErrors = startupErrors + "Unable to to find experimentServiceUrl-node in config.xml.\r\n";
					Error.log(e);
				}

				// Get resultUrlHeader from config.xml
				try
				{
					xmlNode = xmlDoc.SelectSingleNode("settings/resultUrlHeader"); 
					resultUrlHeader = xmlNode.InnerText;
				}
				catch (Exception e)
				{
					startupErrors = startupErrors + "Unable to to find resultUrlHeader-node in config.xml.\r\n";
					Error.log(e);
				}

				// Get pollingRate from config.xml
				try
				{
					xmlNode = xmlDoc.SelectSingleNode("settings/pollingRate"); 
					pollingRate = int.Parse(xmlNode.InnerText);
					if (pollingRate < 1000)
					{
						pollingRate = 1000;
					}
				}
				catch (Exception e)
				{
					startupErrors = startupErrors + "Error while reading pollingRate-node in config.xml.\r\n";
					Error.log(e);
				}

				// Get cameranames and urls from config.xml
				try 
				{
					xmlNode = xmlDoc.SelectSingleNode("settings/cameras");
					if (xmlNode != null)
					{
						xmlNode = xmlNode.FirstChild;
					}

					// Read all child-nodes of settings/cameras 
					while (xmlNode != null)
					{
						if (xmlNode.Name == "camera")
						{
							string name = null;
							string url = null;
							XmlNode xmlChildNode = xmlNode.FirstChild;
							
							// Read all child-nodes of the found camera-node for the name and url
							while (xmlChildNode != null)
							{
								if (xmlChildNode.Name == "name") 
								{
									name = xmlChildNode.InnerText;
								}
								xmlChildNode = xmlChildNode.NextSibling;
								if (xmlChildNode.Name == "url")
								{
									url = xmlChildNode.InnerText;
								}
								if (name != "" && url != "" && cameraNames.IndexOf(name) == -1)
								{
									// Found a new camera									
									cameraNames.Add(name);									
									cameraUrls.Add(url);
									break;
								}
								xmlChildNode = xmlChildNode.NextSibling;
							}
						}
						xmlNode = xmlNode.NextSibling;
					}		
				}
				catch 
				{
					startupErrors = startupErrors + "Error while reading cameras-node in config.xml.\r\n";
				}

			}
			catch (System.Exception e)
			{
				startupErrors = startupErrors + "Unable to open config.xml.\r\n";
				Error.log(e);
			}
		}
		

		public string checkConnection()
		{
			bool check = true;
			string errorText = null;

			// try to connect to DCL by getting an experimentlist
			string[] testExperiments = null;
			try 
			{
				testExperiments = experimentService.ListExperiments ();
			}
			catch
			{
				errorText = "Connection to DCL:\r\n" +
					        "ExperimentService is not reachable.\r\n" + 
							"Please check your VPN connection.";
				check = false;
			}
						

			// Check username and password by getting a testticket
			string testTicket = null;
			if (check == true)
			{
				try
				{
					testTicket = ticketServer.GetNewTicket(txtUsername.Text, txtPassword.Text);
					
					if (testTicket == "S_ERROR_INVALID_LOGIN")
					{
						// wrong login-information
						errorText = "Connection to DCL:\r\n" +
							        "Invalid username or password.";
						check = false;
					}
					else
					{
						try
						{
							ticketServer.FreeTicket(testTicket);
						}
						catch {}
					}
				}
				catch (Exception e)
				{
					errorText = "Connection to DCL:\r\n" +
						        "Unknown error. For further information see error.log";
					Error.log(e);
					check = false;
                }
			}

			return errorText;			
		}


		private void checkConnectionThread ()
		{
			txtError.Text = startupErrors + "Connection to DCL:\r\n" +
				"testing ...";

			string errorText = checkConnection();
			if (errorText == null)
			{
				txtError.Text = startupErrors + "Connection to DCL:\r\n" +
					"Connected successfully.";
			}
			else 
			{
				txtError.Text = startupErrors + errorText;
			}
		}


		
		// Clean up any resources being used.
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}


		#region Windows Form Designer generated code
		
		// Required method for Designer support - do not modify
		// the contents of this method with the code editor.
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(Settings));
			this.txtUsername = new System.Windows.Forms.TextBox();
			this.txtPassword = new System.Windows.Forms.TextBox();
			this.btmTest = new System.Windows.Forms.Button();
			this.btmOk = new System.Windows.Forms.Button();
			this.btmCancel = new System.Windows.Forms.Button();
			this.lblUsername = new System.Windows.Forms.Label();
			this.lblPassword = new System.Windows.Forms.Label();
			this.txtError = new System.Windows.Forms.TextBox();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.groupBox2.SuspendLayout();
			this.SuspendLayout();
			// 
			// txtUsername
			// 
			this.txtUsername.Location = new System.Drawing.Point(96, 40);
			this.txtUsername.Name = "txtUsername";
			this.txtUsername.Size = new System.Drawing.Size(152, 20);
			this.txtUsername.TabIndex = 0;
			this.txtUsername.Text = "";
			// 
			// txtPassword
			// 
			this.txtPassword.Location = new System.Drawing.Point(96, 72);
			this.txtPassword.Name = "txtPassword";
			this.txtPassword.PasswordChar = '*';
			this.txtPassword.Size = new System.Drawing.Size(152, 20);
			this.txtPassword.TabIndex = 1;
			this.txtPassword.Text = "";
			// 
			// btmTest
			// 
			this.btmTest.Location = new System.Drawing.Point(120, 24);
			this.btmTest.Name = "btmTest";
			this.btmTest.Size = new System.Drawing.Size(112, 24);
			this.btmTest.TabIndex = 2;
			this.btmTest.Text = "&Check connection";
			this.btmTest.Click += new System.EventHandler(this.btmTest_Click);
			// 
			// btmOk
			// 
			this.btmOk.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.btmOk.Location = new System.Drawing.Point(16, 288);
			this.btmOk.Name = "btmOk";
			this.btmOk.Size = new System.Drawing.Size(112, 24);
			this.btmOk.TabIndex = 3;
			this.btmOk.Text = "&OK";
			this.btmOk.Click += new System.EventHandler(this.btmOk_Click);
			// 
			// btmCancel
			// 
			this.btmCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btmCancel.Location = new System.Drawing.Point(152, 288);
			this.btmCancel.Name = "btmCancel";
			this.btmCancel.Size = new System.Drawing.Size(112, 24);
			this.btmCancel.TabIndex = 4;
			this.btmCancel.Text = "&Cancel";
			this.btmCancel.Click += new System.EventHandler(this.btmCancel_Click);
			// 
			// lblUsername
			// 
			this.lblUsername.Location = new System.Drawing.Point(32, 40);
			this.lblUsername.Name = "lblUsername";
			this.lblUsername.Size = new System.Drawing.Size(72, 24);
			this.lblUsername.TabIndex = 5;
			this.lblUsername.Text = "username";
			// 
			// lblPassword
			// 
			this.lblPassword.Location = new System.Drawing.Point(32, 72);
			this.lblPassword.Name = "lblPassword";
			this.lblPassword.Size = new System.Drawing.Size(72, 24);
			this.lblPassword.TabIndex = 6;
			this.lblPassword.Text = "password";
			// 
			// txtError
			// 
			this.txtError.Location = new System.Drawing.Point(32, 176);
			this.txtError.Multiline = true;
			this.txtError.Name = "txtError";
			this.txtError.ReadOnly = true;
			this.txtError.Size = new System.Drawing.Size(216, 83);
			this.txtError.TabIndex = 7;
			this.txtError.TabStop = false;
			this.txtError.Text = "";
			// 
			// groupBox1
			// 
			this.groupBox1.Location = new System.Drawing.Point(16, 16);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(248, 88);
			this.groupBox1.TabIndex = 9;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "user settings";
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.Add(this.btmTest);
			this.groupBox2.Location = new System.Drawing.Point(16, 120);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(248, 152);
			this.groupBox2.TabIndex = 10;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "status";
			// 
			// Settings
			// 
			this.AcceptButton = this.btmOk;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.btmCancel;
			this.ClientSize = new System.Drawing.Size(282, 328);
			this.Controls.Add(this.txtError);
			this.Controls.Add(this.txtPassword);
			this.Controls.Add(this.txtUsername);
			this.Controls.Add(this.lblPassword);
			this.Controls.Add(this.lblUsername);
			this.Controls.Add(this.btmCancel);
			this.Controls.Add(this.btmOk);
			this.Controls.Add(this.groupBox1);
			this.Controls.Add(this.groupBox2);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "Settings";
			this.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.ShowInTaskbar = false;
			this.Text = "DCL VSI - Settings";
			this.groupBox2.ResumeLayout(false);
			this.ResumeLayout(false);

		}
	
		#endregion

		private void btmOk_Click(object sender, System.EventArgs e)
		{
			if (username != txtUsername.Text | password != txtPassword.Text)
			{
				// abort the testing thread if running
				try
				{
					testThread.Abort();
				}
				catch{}

				// user changed entries of username and password
				username = txtUsername.Text;
				password = txtPassword.Text;
				writeRegistry();

				// Parameter is unused
				dclvsi.refreshMenuAndJobExplorer(-1);
			}
			this.Close();
		}


		private void btmCancel_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}


		private void btmTest_Click(object sender, System.EventArgs e)
		{
			// abort the testing thread if running
			try
			{
				testThread.Abort();
			}
			catch{}

			// start new thread
			testThread = new Thread(new ThreadStart(this.checkConnectionThread));
			testThread.Start();
		}
	}
}
